Index: apps/app_meetme.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_meetme.c,v
retrieving revision 1.96
diff -u -u -r1.96 app_meetme.c
--- apps/app_meetme.c	29 Apr 2005 17:00:33 -0000	1.96
+++ apps/app_meetme.c	4 May 2005 20:40:50 -0000
@@ -144,12 +144,15 @@
 	char usrvalue[50];		 /* Custom User Value */
 	char namerecloc[AST_MAX_EXTENSION]; /* Name Recorded file Location */
 	time_t jointime;		 /* Time the user joined the conference */
+	int volume;
 };
 
 #define ADMINFLAG_MUTED (1 << 1)	/* User is muted */
 #define ADMINFLAG_KICKME (1 << 2)	/* User is kicked */
 #define MEETME_DELAYDETECTTALK 		300
 #define MEETME_DELAYDETECTENDTALK 	1000
+#define MEETME_VOL_UP 1
+#define MEETME_VOL_DOWN -1
 
 AST_MUTEX_DEFINE_STATIC(conflock);
 
@@ -249,6 +252,41 @@
 	return 0;
 }
 
+#define minmax(x,y) x ? (x > y) ? y : ((x < (y * -1)) ? (y * -1) : x) : 0
+
+static void tweak_volume(struct ast_conf_user *user, int sense)
+{
+	
+	if (sense == MEETME_VOL_UP) {
+		user->volume++;
+		if (abs(user->volume) == 1) {
+			user->volume++;
+		}
+	} else if(sense == MEETME_VOL_DOWN) {
+		user->volume--;
+		if (abs(user->volume) == 1) {
+			user->volume--;
+		}
+	}
+
+	user->volume = minmax(user->volume, 5);
+
+}
+
+static void adjust_volume(struct ast_frame *f, int vol) {
+	int count = 0;
+	short *fdata = f->data;
+
+	for(count=0; count < f->datalen; count++) {
+		if (vol > 0) {
+			fdata[count] *= abs(vol);
+		} else if (vol < 0) {
+			fdata[count] /= abs(vol);
+		}
+	}
+}
+
+
 static void conf_play(struct ast_channel *chan, struct ast_conference *conf, int sound)
 {
 	unsigned char *data;
@@ -1035,6 +1073,10 @@
 				if (!f)
 					break;
 				if ((f->frametype == AST_FRAME_VOICE) && (f->subclass == AST_FORMAT_SLINEAR)) {
+					if (user->volume) {
+						adjust_volume(f, user->volume);
+					}
+
 					if (confflags &  CONFFLAG_MONITORTALKER) {
 						int totalsilence;
 						if (user->talking == -1)
@@ -1145,6 +1187,19 @@
 										usr->adminflags |= ADMINFLAG_KICKME;
 									ast_stopstream(chan);
 									break;	
+
+								case '9':
+									tweak_volume(user, MEETME_VOL_UP);
+									break;
+
+								case '8':
+									menu_active = 0;
+									break;
+
+								case '7':
+									tweak_volume(user, MEETME_VOL_DOWN);
+									break;
+
 								default:
 									menu_active = 0;
 									/* Play an error message! */
@@ -1188,6 +1243,18 @@
 											ast_waitstream(chan, "");
 									}
 									break;
+							case '9':
+								tweak_volume(user, MEETME_VOL_UP);
+								break;
+
+							case '8':
+								menu_active = 0;
+								break;
+
+							case '7':
+								tweak_volume(user, MEETME_VOL_DOWN);
+								break;
+
 								default:
 									menu_active = 0;
 									/* Play an error message! */
